/**
 * Created by Tony on 2021/9/10.
 */

import BaseController from "../../common/base/base_controller";
import featureFacade from "../facade/bills_unitprice_feature_facade";
let config = require("../../../config/config.js");
import CompilationModel from '../../users/models/compilation_model';
import { checkCompilationPermission } from '../../common/base/base_util';

class BillsUnitPriceFeatureController extends BaseController {
    async main(request, response) {
        let compilationModel = new CompilationModel();
        let compilationList = await compilationModel.getPermissionCompilationList(request, { _id: 1, name: 1 });
        compilationList.unshift({ _id: 'all', name: '所有' });
        let activeCompilation = compilationList.find(compilation => compilation._id.toString() === request.query.filter);
        if (activeCompilation) {
            activeCompilation.active = 'active';
        } else {
            compilationList[0].active = 'active'
        }
        let filter = request.query.filter ? { compilationId: request.query.filter } : {};
        let featureLibs = await featureFacade.findByCondition(filter, { feature: 0 }, false);
        const compilationPermission = request.session.managerData.compilationPermission || [];
        featureLibs = featureLibs.filter(lib => compilationPermission.includes(lib.compilationId));
        let randerData = {
            title: '清单估算指标库',
            userAccount: request.session.managerData.username,
            userID: request.session.managerData.userID,
            featureLibs: featureLibs,
            compilationList: compilationList,
            layout: 'maintain/common/html/layout'
        };
        response.render("maintain/bills_unitprice_feature_lib/html/main", randerData);
    }
    async addLib(request, response) {
        try {
            await featureFacade.addLib(request.body);
        } catch (error) {
            console.log(error);
        }
        response.redirect(request.headers.referer);
    }
    async findLib(request, response) {
        let result = {
            error: 0
        };
        try {
            let data = request.body.data;
            data = JSON.parse(data);
            let conditions = { 'ID': data.ID };
            let resultData = await featureFacade.findByCondition(conditions);
            result.data = resultData;
        } catch (err) {
            console.log(err);
            result.error = 1;
            result.message = err.message;
        }
        response.json(result);
    }
    async saveLib(request, response) {
        let result = {
            error: 0
        };
        try {
            let data = request.body.data;
            data = JSON.parse(data);
            let resultData = await featureFacade.saveLib(data);
            result.data = resultData;
        } catch (err) {
            console.log(err);
            result.error = 1;
            result.message = err.message;
        }
        response.json(result);
    }
    async deleteLibByID(request, response) {
        let result = {
            error: 0
        };
        try {
            let data = request.body.data;
            data = JSON.parse(data);
            let resultData = await featureFacade.deleteLibByID(data.ID);
            result.data = resultData;
        } catch (err) {
            console.log(err);
            result.error = 1;
            result.message = err.message;
        }
        response.json(result);
    }
    async edit(request, response) {
        //先取出替换库信息：
        let libID = request.params.libID;
        let featureLib = await featureFacade.findByCondition({ 'ID': libID });
        if (featureLib) {
            checkCompilationPermission(request, response, featureLib.compilationId, '/billsUnitPriceFeature/main');
            let randerData = {
                title: '清单估算指标库',
                mainURL: '/billsUnitPriceFeature/main',
                libName: featureLib.name,
                userAccount: request.session.managerData.username,
                userID: request.session.managerData.userID,
                featureList: JSON.stringify(featureLib.feature),
                libID: libID,
                LicenseKey: config.getLicenseKey(process.env.NODE_ENV),
                layout: 'maintain/common/html/edit_layout'
            };
            response.render("maintain/bills_unitprice_feature_lib/html/edit", randerData);
        } else {
            response.redirect(request.headers.referer);
        }
    }
}
export default BillsUnitPriceFeatureController;